@@ -72,7 +72,7 @@ namespace TagLib {
 	///    variants, as it automatically detects the appropriate class from
 	///    the file extension or provided mime-type.</para>
 	/// </remarks>
-	public abstract class File
+	public abstract class File : IDisposable
 	{
 		#region Enums
 		
@@ -106,7 +106,7 @@ namespace TagLib {
 		
 		/// <summary>
 		///    This delegate is used for intervening in <see
-		///    cref="Create(string)" /> by resolving the file type
+		///    cref="File.Create(string)" /> by resolving the file type
 		///    before any standard resolution operations.
 		/// </summary>
 		/// <param name="abstraction">
@@ -127,17 +127,17 @@ namespace TagLib {
 		/// </returns>
 		/// <remarks>
 		///    <para>A <see cref="FileTypeResolver" /> is one way of
-		///    altering the behavior of <see cref="Create(string)"
-		///    />.</para>
-		///    <para>When <see cref="Create(string)" /> is called, the
+		///    altering the behavior of <see cref="File.Create(string)" />
+		///    .</para>
+		///    <para>When <see cref="File.Create(string)" /> is called, the
 		///    registered resolvers are invoked in the reverse order in
 		///    which they were registered. The resolver may then perform
 		///    any operations necessary, including other type-finding
 		///    methods.</para>
 		///    <para>If the resolver returns a new <see cref="File" />,
 		///    it will instantly be returned, by <see
-		///    cref="Create(string)" />. If it returns <see 
-		///    langword="null" />, <see cref="Create(string)" /> will
+		///    cref="File.Create(string)" />. If it returns <see 
+		///    langword="null" />, <see cref="File.Create(string)" /> will
 		///    continue to process. If the resolver throws an exception
 		///    it will be uncaught.</para>
 		///    <para>To register a resolver, use <see
@@ -197,6 +197,12 @@ namespace TagLib {
 		///    the file ends.
 		/// </summary>
 		private long invariant_end_position = -1;
+
+		/// <summary>
+		///    The reasons (if any) why this file is marked as corrupt.
+		/// </summary>
+		private List<string> corruption_reasons = null;
+
 		#endregion
 		
 		
@@ -441,11 +447,66 @@ namespace TagLib {
 			}
 		}
 		
+		/// <summary>
+		///    Indicates if tags can be written back to the current file or not
+		/// </summary>
+		/// <value>
+		///    A <see cref="bool" /> which is true if tags can be written to the
+		///    current file, otherwise false.
+		/// </value>
+		public virtual bool Writeable {
+			get { return !PossiblyCorrupt; }
+		}
+
+		/// <summary>
+		///   Indicates whether or not this file may be corrupt.
+		/// </summary>
+		/// <value>
+		/// <c>true</c> if possibly corrupt; otherwise, <c>false</c>.
+		/// </value>
+		/// <remarks>
+		///    Files with unknown corruptions should not be written.
+		/// </remarks>
+		public bool PossiblyCorrupt {
+			get { return corruption_reasons != null; }
+		}
+
+		/// <summary>
+		///   The reasons for which this file is marked as corrupt.
+		/// </summary>
+		public IEnumerable<string> CorruptionReasons {
+			get {
+				return corruption_reasons;
+			}
+		}
+
 		#endregion
 		
 		
 		
 		#region Public Methods
+
+		/// <summary>
+		///	   Mark the file as corrupt.
+		/// </summary>
+		/// <param name="reason">
+		///    The reason why this file is considered to be corrupt.
+		/// </param>
+		internal void MarkAsCorrupt (string reason)
+		{
+			if (corruption_reasons == null)
+				corruption_reasons = new List<string> ();
+			corruption_reasons.Add (reason);
+		}
+
+		/// <summary>
+		///    Dispose the current file. Equivalent to setting the
+		///    mode to closed
+		/// </summary>
+		public void Dispose ()
+		{
+			Mode = AccessMode.Closed;
+		}
 		
 		/// <summary>
 		///    Saves the changes made in the current instance to the
@@ -604,15 +665,18 @@ namespace TagLib {
 			
 			Mode = AccessMode.Read;
 			
-			if (Tell + length > Length)
-				length = (int) (Length - Tell);
-			
-			if (length <= 0)
-				return new ByteVector ();
-			
 			byte [] buffer = new byte [length];
-			int count = file_stream.Read (buffer, 0, length);
-			return new ByteVector (buffer, count);
+
+			int count = 0, read = 0, needed = length;
+
+			do {
+				count = file_stream.Read (buffer, read, needed);
+
+				read += count;
+				needed -= count;
+			} while(needed > 0 && count != 0);
+
+			return new ByteVector (buffer, read);
 		}
 		
 		/// <summary>
@@ -885,10 +949,10 @@ namespace TagLib {
 			
 			// These variables are used to keep track of a partial
 			// match that happens at the end of a buffer.
-			
+
 			/*
 			int previous_partial_match = -1;
-			int before_previous_partial_match = -1;
+			int after_previous_partial_match = -1;
 			*/
 			
 			// Save the location of the current read pointer.  We
@@ -899,22 +963,18 @@ namespace TagLib {
 			
 			// Start the search at the offset.
 			
-			long buffer_offset;
+			long buffer_offset = Length - startPosition;
+			int read_size = buffer_size;
 			
-			if (startPosition == 0)
-				Seek (-1 * buffer_size,
-					System.IO.SeekOrigin.End);
-			else
-				Seek (startPosition - 1 * buffer_size,
-					System.IO.SeekOrigin.Begin);
-			
-			buffer_offset = file_stream.Position;
+			read_size = (int) Math.Min (buffer_offset, buffer_size);
+			buffer_offset -= read_size;
+			file_stream.Position = buffer_offset;
 			
 			// See the notes in find() for an explanation of this
 			// algorithm.
 			
-			for (buffer = ReadBlock(buffer_size); buffer.Count > 0;
-				buffer = ReadBlock (buffer_size)) {
+			for (buffer = ReadBlock (read_size); buffer.Count > 0;
+				buffer = ReadBlock (read_size)) {
 				
 				// TODO: (1) previous partial match
 				
@@ -932,8 +992,11 @@ namespace TagLib {
 				}
 				
 				// TODO: (3) partial match
+
 				
-				buffer_offset -= buffer_size;
+				read_size = (int) Math.Min (buffer_offset, buffer_size);
+				buffer_offset -= read_size;
+
 				file_stream.Position = buffer_offset;
 			}
 			
@@ -1769,4 +1832,4 @@ namespace TagLib {
 		
 		#endregion
 	}
-}
\ No newline at end of file
+}
